# Microeconometrics
# CLASS 2
# Continous variables
# Wiktor Budzinski, Marek Giergiczny

rm(list=ls())

# install.packages("readxl") # Install packages if necessary 

library(readxl)
library(rcompanion)
library(jtools)
library(lmtest)
library(tseries)
library(MASS) 
library(splines)
library(glmnet)
library(car)
library(mgcv)
library(statmod)
library(stargazer)
library(mltools)
library(quantreg)
library(bootstrap)
library(lmboot)

# Necessary transformations from the previous class
setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 1")       # sets the access path (sometimes change the characters '\' to '/' is needed)
data <- read_excel("wine.xlsx")
data$Cases <- data$Cases/100 # Scalling the covariate
data$LogPrice <- log(data$Price)
data$LogCases <- log(data$Cases)
data$LogScore <- log(data$Score)
data$ScoreSq <- (data$Score/10)^2
Eq7 <- LogPrice ~ LogCases + LogCases:washington + LogCases:Nonvarietal + LogCases:Pinotnoir + LogCases:Cabernet + LogCases:Merlot + LogCases:Shyrah +
  Score + ScoreSq + Score:washington + Score:Nonvarietal + Score:Pinotnoir + Score:Cabernet + Score:Merlot + Score:Shyrah +
  Age + Age:washington + Age:Nonvarietal + Age:Pinotnoir + Age:Cabernet + Age:Merlot + Age:Shyrah +
  NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99
model7 <- lm(Eq7, data=data)
summ(model7, digits = 4)

################################################# Exercise 1 ########################################
# GLM

# Basic GLM with normal distribution:
glm1 <- glm(Eq7, family = gaussian, data=data)
summ(glm1, digits = 4)

# Different model
Eq11 <- Price ~ LogCases + LogCases:washington + LogCases:Nonvarietal + LogCases:Pinotnoir + LogCases:Cabernet + LogCases:Merlot + LogCases:Shyrah +
  Score + ScoreSq + Score:washington + Score:Nonvarietal + Score:Pinotnoir + Score:Cabernet + Score:Merlot + Score:Shyrah +
  Age + Age:washington + Age:Nonvarietal + Age:Pinotnoir + Age:Cabernet + Age:Merlot + Age:Shyrah +
  NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99
glm2 <- glm(Eq11, family = gaussian(link="log"), data=data)
summ(glm2, digits = 4)

## Different families of distibutions
glm3 <- glm(Eq11, family = Gamma(link="log"), data=data)
summ(glm3, digits = 4)

glm4 <- glm(Eq11, family = inverse.gaussian(link="log"), data=data)
summ(glm4, digits = 4)

## Something wrong?
extractAIC(glm1); extractAIC(glm2); extractAIC(glm3);  extractAIC(glm4);


#data$pred1<-exp(model7$fitted.values+0.5*var(model7$residuals))
data$pred1<-exp(model7$fitted.values+0.5*sigma(model7)^2)
data$pred2<-predict(glm2, type = "response")
data$pred3<-predict(glm3, type = "response")
data$pred4<-predict(glm4, type = "response")

data$err1<- (data$Price - data$pred1)^2
data$err2<- (data$Price - data$pred2)^2
data$err3<- (data$Price - data$pred3)^2
data$err4<- (data$Price - data$pred4)^2

# MSE
mean(data$err1); mean(data$err2); mean(data$err3); mean(data$err4);

# MSE
median(data$err1);median(data$err2);  median(data$err3); median(data$err4)

################################################# Exercise 2 ########################################
# Assumptions testing

data$eP <- resid(glm3, type="pearson" )
data$eD <- resid(glm3) # Deviance resids are the default
data$eQ <- qresid(glm3) # quantile residuals

# Quantile residuals can be used to checked whether assumed distribution is correct

qqnorm(data$eQ, las=1 ); qqline(data$eQ)
# Same thing for the inverse normal
data$eQ2 <- qresid(glm4) # quantile residuals
qqnorm(data$eQ2, las=1 ); qqline(data$eQ2)

## Plotting fitted values against residuals could inform us about functional form

scatter.smooth(data$eD ~ glm3$fitted.values, col="grey",
               las=1, ylab="Deviance residuals", xlab="XB")
scatter.smooth(data$eP ~ glm3$fitted.values, col="grey",
               las=1, ylab="Pearson's residuals", xlab="XB")

data$eP2 <- resid(glm4, type="pearson" )
data$eD2 <- resid(glm4) # Deviance resids are the default
scatter.smooth(data$eD2 ~ glm4$fitted.values, col="grey",
               las=1, ylab="Deviance residuals", xlab="XB")
scatter.smooth(data$eP2 ~ glm4$fitted.values, col="grey",
               las=1, ylab="Pearson's residuals", xlab="XB")

# Individual covariates could be plotted as well
scatter.smooth(data$eP2 ~ data$Cases, col="grey",
               las=1, ylab="Pearson's residuals", xlab="Cases")


# Of course we could try to calculate RESET test by hand
data$xb2 <-glm4$fitted.values^2
data$xb3 <-glm4$fitted.values^3

Eq11b <- Price ~ LogCases + LogCases:washington + LogCases:Nonvarietal + LogCases:Pinotnoir + LogCases:Cabernet + LogCases:Merlot + LogCases:Shyrah +
  Score + ScoreSq + Score:washington + Score:Nonvarietal + Score:Pinotnoir + Score:Cabernet + Score:Merlot + Score:Shyrah +
  Age + Age:washington + Age:Nonvarietal + Age:Pinotnoir + Age:Cabernet + Age:Merlot + Age:Shyrah +
  NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99+xb2+xb3
glm4b <- glm(Eq11b, family = gaussian(link="log"), data=data)
summ(glm4b, digits = 4)
linearHypothesis(model=glm4b, c("xb2=0","xb3=0")) # Test RESET

im <- influence.measures(glm4)
colSums(im$is.inf)

################################################# Exercise 3 ########################################
# Quantile regression

# rm(list=ls())
# ctrl+l

help(rq)
# Quantile regression (command 'rq')
Eq7x <- LogPrice ~ LogCases + Score + Age +
  NapaValley+BayArea+sonoma+SouthCoast+carneros+Sierra+mendocino + washington +
  Nonvarietal + Pinotnoir + Cabernet + Merlot + Shyrah + Reserve + Vineyard + Estate +
  Vintage91 + Vintage92 + Vintage93 + Vintage94 + Vintage95 + Vintage96  + Vintage97 + Vintage98 + Vintage99

m50 <- rq(Eq7x, data=data)             # median regression (default)
summary(m50)
m50_2<- rq(Eq7x, data=data, tau=0.50)      # for quantile 0.50 (median)
summary(m50_2)

m25<- rq(Eq7x, data=data, tau=0.25)      # for quantile 0.25
summary(m25)
m75<-  rq(Eq7x, data=data, tau=0.75)      # for quantile 0.75
summary(m75)

# Display results in one table
stargazer(m25, m50, m75, type="text", df=FALSE)

# You can compute all the regressions above with one command
modeljoint <- rq(Eq7x, data=data, tau=c(0.25,0.50,0.75))
summary(modeljoint)

# Test for coefficient differences (ANOVA)
anova(m25, m75)

help(boot.rq)                 # bootstrapping quantile regression (from library quantreg)
# For boot.rq function, standard errors are calculated using the bootstrap method.
# 'R=' sets how many repetitions this method will use
summary(m50)   # for comparison
summary(m50, se="boot", R=200)

################################################# Exercise 4 ########################################

# Estimate quantile regression for 0.1, 0.25, 0.5, 0.75 and 0.9 quantiles (using rq command)

m10 <- rq(Eq7x, data=data, tau=0.1)
m90 <- rq(Eq7x, data=data, tau=0.9)

stargazer(m10, m25, m50, m75, m90, type="text", df=FALSE)

# Test whether quantile regression indicates heteroskedasticity

# Test for coefficient differences (ANOVA)   # ANOVA test can not be computed for object with class 'boot' (in R)  (default test statistic: Wald's)
anova(m10, m25, m50, m75, m90)

# Anova test for quantile regression fits (separately variables)
anova.rq(m10, m25, m50, m75, m90, joint=FALSE)

# Peform graphical analysis. Check whether the parameters change between quantiles.

mjoint2 <- rq(Eq7x, data=data,tau=c(0.1,0.25,0.5,0.75,0.9))
quantile_plot <- summary(mjoint2)
plot(quantile_plot)

