library(readxl)
library(rcompanion)
library(jtools)
library(lmtest)
library(tseries)
library(MASS) 
library(splines)
library(glmnet)
library(car)
library(mgcv)
library(statmod)
library(stargazer)
library(mltools)
library(quantreg)


## Simulation Example no 1.

set.seed(1234567) # setting seed

eps<-rnorm(5000, 0, 1)
x1 <- (runif(5000,0,1) > 0.5)
sigma <- exp(-1.5*x1)
y <- exp(0.2 + 0.5*x1 + sigma*eps)
hist(y)

logy <- log(y)

m1<-lm(logy~x1)
summ(m1, digits = 4)

m2 <- glm(y~x1, family = Gamma(link="log"))
summ(m2, digits = 4)

m3 <- glm(y~x1, family = inverse.gaussian(link="log"))
summ(m3, digits = 4)

pred1<-exp(m1$fitted.values+0.5*var(m1$residuals))
pred2<-predict(m2, type = "response")
pred3<-predict(m3, type = "response")

err1<- (y - pred1)^2
err2<- (y - pred2)^2
err3<- (y - pred3)^2

# MSE
mean(err1); mean(err2); mean(err3);
median(err1); median(err2); median(err3)

## Simulation Example no 2
# Quantile regression

rm(list=ls())
set.seed(1234567) # setting seed

eps<-rnorm(5000, 0, 1)
x1 <- (runif(5000,0,1) > 0.5)
x2 <- rnorm(5000,0,1)
x3 <- rnorm(5000,0,1)

sigma <- exp(-1.5*x1 + x3)
y <- 0.2 + 0.5*x1 - x2 + sigma*eps
hist(y)
m0<-lm(y~ x1 + x2 + x3)
summary(m0)
summ(m0, digits = 4, robust = "HC0")

m10 <- rq(y~ x1 + x2 + x3, tau=0.1)
m25 <- rq(y~ x1 + x2 + x3, tau=0.25)
m50 <- rq(y~ x1 + x2 + x3, tau=0.5)
m75 <- rq(y~ x1 + x2 + x3, tau=0.75)
m90 <- rq(y~ x1 + x2 + x3, tau=0.9)
stargazer(m10, m25, m50, m75, m90, type="text", df=FALSE)
anova.rq(m10, m25, m50, m75, m90, joint=FALSE)

mx <- rq(y~ x1 + x2 + x3, tau=c(0.1,0.25,0.5,0.75,0.9))
quantile_plot <- summary(mx)
plot(quantile_plot)
